#include <iostream>
#include <vector>


using namespace std;

void dfs(const vector<vector<int>>& graph, vector<int> &components, int v, int usedS)
{
	components[v] = usedS;
	for (int i = 0; i < graph[v].size(); i++)
		if (components[graph[v][i]] == -1)
			dfs(graph, components, graph[v][i], usedS);
}


int main()
{
	ios::sync_with_stdio();
	cin.tie();
	cout.tie();
	int N, M, K;
	cin >> N >> M >> K;

	vector<int> components;
	vector<pair<int, int>> input;
	components.resize(N, -1);

	for (int i = 0; i < M; i++)
	{
		int u, v;
		cin >> u >> v;
		input.push_back({ u, v });
	}

	//O(K)
	vector<vector<int>> graph;
	graph.resize(N);
	for (int i = 0; i < K; i++)
	{
		int u, v;
		cin >> u >> v;
		graph[u - 1].push_back(v - 1);
		graph[v - 1].push_back(u - 1);
	}

	//O(N + M)
	int total_components = 0;
	for (int i = 0; i < N; i++)
		if (components[i] == -1)
			dfs(graph, components, i, total_components++);

	//O(N^2)
	vector<vector<int>> friends; //how much friend V have IN G
	friends.resize(N);
	for (int i = 0; i < N; i++)
		friends[i].resize(total_components + 1, 0);

	//O(K)
	for (int i = 0; i < M; i++)
	{
		int u = input[i].first;
		int v = input[i].second;
		friends[v - 1][components[u - 1]]++;
		friends[u - 1][components[v - 1]]++;
	}

	//O(NQ)
	int Q;
	cin >> Q;
	for (int i = 0; i < Q; i++)
	{
		char mode;
		cin >> mode;
		if (mode == '?')
		{
			int v;
			cin >> v;
			cout << friends[v - 1][components[v - 1]] << endl;
		}
		else if (mode == 'F')
		{
			int u, v;
			cin >> u >> v;
			friends[v - 1][components[u - 1]]++;
			friends[u - 1][components[v - 1]]++;
		}
		else if (mode == 'T')
		{
			int u, v;
			cin >> u >> v;
			int Gu = components[u - 1], Gv = components[v - 1];
			if (Gu == Gv)
				continue;
			for (int i = 0; i < N; i++)
			{
				friends[i][Gu] += friends[i][Gv];
				if (components[i] == Gv)
					components[i] = Gu;
			};
		};
	};
	return 0;

}